/*
  Residência Univesitária - Exemplo de Agregação
  A classe "Residencia" vai usar n (vectores) instâncias de "Estudante",
  "EstudanteMestDout", "Empregado e Professor"

  Residencia <>--1-n-- Estudantes
             <>--1-n-- EstudanteMestDout
	     <>--1-n-- Empregados
	     <>--1-n-- Professores
  Residentes <- Estudantes <- EstudanteMestDout
             <- Empregados <- Professores 
*/

#include <iostream>  
#include "residentes.hpp"
#include "estudantes.hpp"
#include "empregados.hpp"

using namespace std; 

class Residencia {
private:
  static const int numeroQuartos = 10;
  static const int numResidentes = 300;
  // número fichas de Residentes
  int numListaEst, numListaEstMstDtr, numListaEmp, numListaPrf;
  Estudante listaEstudantes[numResidentes]; // agregação 1..n
  EstudanteMestDout listaEstudantesMestDout[numResidentes]; // agregação 1..n
  Empregado listaEmpregados[numResidentes]; // agregação 1..n
  Professor listaProfessores[numResidentes]; // agregação 1..n
  // Lista de quartos livres - em cada célula: 0 - livre; 1 - ocupado
  int listaVagas[numeroQuartos]; 
  /*
    Matriz que nos dá a ocupação dos quartos
    Na posição (n,m) o quarto n tem o ocupante m; 
    1 - Estudantes ; 
    2 - EstudantesMestDout
    3 - Empregados
    4 - Professores
    0 - em todos os outros casos
  */
  int quartosOcupados[numeroQuartos][numResidentes]; 
public:
  /*
    Construtor
    -> 
    <- listaVagas e quartosOcupados inicializados a zero
  */
  Residencia() {
    int i,j;
    for (i=0;i<numeroQuartos;i++) {
      listaVagas[i]=0;
      for (j=0;j<numResidentes;j++)
	quartosOcupados[i][j]=0;
    }
  };
  /*
    mostraVagas - quartos livres
    -> objecto
    <- indicação «stdout» dos quartos que estão livres
  */
  void mostraVagas() {
    int i;
    cout << "\nLista de quartos livres: ";
    for (i=0;i<numeroQuartos;i++)
      if (listaVagas[i]==0) cout << i+1 << " ";
    cout << endl;
  };  
  /*
    mostraOcupado - quartos ocupados
    -> objecto
    <- indicação «stdout» dos quartos que estão ocupados
  */  
  void mostraOcupacao() {
    int i;
    cout << "\nLista de quartos ocupados\n\n ";
    for (i=0;i<numeroQuartos;i++)
      if (listaVagas[i]==1) 
	cout << "=== O Quarto "<< i+1 << " está ocupado ===\n";
    cout << endl;
  };   
  /*
    mostraDadosOcupante - mostra dados do ocupante do quarto n
    -> n, número de quarto (1<=n<=numeroQuartos)
    <- dados de ocupante (método da classe respectiva)
  */ 
  void mostraDadosOcupante(int n) {
    int j,tipoOcupante = 0;
    // verifica se o quarto está ocupado
    if (listaVagas[n]==0) 
      cout << "O quarto " << n << " não está ocupado\n\n";
    else {
      // verificar qual é o ocupante e de que tipo
      for (j=0;j<numeroQuartos && tipoOcupante==0;j++)
	if (quartosOcupados[n][j]!=0)
	  tipoOcupante = quartosOcupados[n][j]; 
      switch (tipoOcupante) {
      case 1: // Estudante
	listaEstudantes[j].mostraInfo();
	break;
      case 2: // EstudantesMestDout
	listaEstudantesMestDout[j].mostraInfo();
	break;
      case 3: // Empregados
	listaEmpregados[j].mostraInfo();
	break;
      case 4: // Professores
	listaProfessores[j].mostraInfo();
	break;
      default: // alienígena
	cout << "Erro: tipo ocupante não identificado\n\n";
      }
    }
  };
  /*
    reservaQuarto - reserva um quarto, verificando se o mesmo está livre ou não
    -> tipoOcupante, 1-Estudante; ...
       numOcupante, posição na lista respectiva
       n, 1<=n<=numeroQuartos
    <- listaVagas[n]=1
       quartosOcupados[n][numOcupante]=tipoOcupante
  */
  void reservaQuarto(int tipoOcupante,int numOcupante,int n) {
    if (listaVagas[n]!=0) {
      cout << "O quarto " << n << " está ocupado\n\n";
    }
    else {
      listaVagas[n]=1;
      quartosOcupados[n][numOcupante]=tipoOcupante;
    }
  }; 
  /*
    libertaQuarto - liberta um quarto, não verifica se está ou não ocupado
    -> n, 1<=n<=numeroQuartos
    <- listaVagas[n]=0
       quartosOcupados[n][*]=0
  */  
  void libertaQuarto(int n) {
    int j;
    listaVagas[n]=0;
    for (j=0;j<numResidentes;j++)
      quartosOcupados[n][j]=0;
  };
  /*
    preencheFicha - Preenche ficha de ocupante consuante o tipo
    -> tipoOcupante, 1-Estudante; ...
    <- lista<tipoOcupante>[num<tipoOcupante] = <ocupante>
       num<tipoOcupante>++
       é dada a informação da posição ocupada e são ecoados os dados introduzidos
  */
  int preencheFicha(int tipoOcupante) {
    int i;
    string nomeAux,depAux,nomePrgAux,supAux,espAux;
    unsigned long ccAux,numAux,numOrientAux;
    switch (tipoOcupante) {
    case 1: // Estudante
      cout << "Introduza dados de Estudante (nome,cc,dep,num): ";
      cin >> nomeAux >>  ccAux >> depAux >> numAux;
      listaEstudantes[numListaEst].fixaInfo(nomeAux,ccAux,depAux,numAux);
      cout << "Novo residente (estudante) na posição: " << numListaEst << endl;
      listaEstudantes[numListaEst].mostraInfo();
      numListaEst++;
      break;
    case 2: // EstudantesMestDout
      cout << "Introduza dados de Estudante de Mestradro ou Doutoramento: ";
      cin >> nomeAux >> depAux >> ccAux >> numAux;
      listaEstudantesMestDout[numListaEstMstDtr].fixaInfo(nomeAux,ccAux,depAux,numAux,nomePrgAux,supAux);
      numListaEstMstDtr++;
      break;
    case 3: // Empregados
      cout << "Introduza dados de Empregado: ";
      cin >> nomeAux >> ccAux >> numAux;
      listaEmpregados[numListaEmp].fixaInfo(nomeAux,ccAux,numAux);
      numListaEmp++;
      break;
    case 4: // Professores
      cout << "Introduza dados de Professor: ";
      cin >> nomeAux >> ccAux >> numAux >> espAux >> numOrientAux;
      listaProfessores[numListaPrf].fixaInfo(nomeAux,ccAux,numAux,espAux,numOrientAux);
      numListaPrf++;
      break;
    default: // alienígena
      cout << "Erro: tipo ocupante não identificado\n\n";
    }
    return(numListaPrf-1);
  };
};


int main() {
  
  string nome,especializacao;
  unsigned long cc,numEmpr;
  unsigned int numOrientacoes;
  int pos,numQuarto;

  // Cria uma residência com 10 quartos e 300x4 possíveis residentes
  Residencia residencia;

  // mostra lista de quartos vazios
  residencia.mostraVagas();

  // vai preencher uma ficha de residente para um estudante
  cout << "Prenche ficha de residente (estudante)\n\n";
  pos = residencia.preencheFicha(1);
  
  // tipo, posição na lista, numQuarto
  cout << "\nNúmero do quarto que pretende ocupar? ";
  cin >> numQuarto;
  residencia.reservaQuarto(1,pos,numQuarto-1);

  // mostra lista de quartos ocupados
  residencia.mostraOcupacao();

  return 0;
}



